import sys

YES = ["y", "Y", "yes", "Yes", "YES"]
NO = ["n", "N", "no", "No", "NO"]


def ask(msg: str) -> str:
    print(msg, end="")
    return input()


def is_yes(r: str) -> bool:
    return r in YES


def is_no(r: str) -> bool:
    return r in NO


def is_yes_or_no(r: str) -> bool:
    return is_yes(r) or is_no(r)


def is_empty(r: str) -> bool:
    return r == ""


def can_be_int(r: str) -> bool:
    try:
        int(r)
        return True
    except ValueError:
        return False


def ask_with_default(msg: str, default: str) -> str:
    r = ask(msg)
    return default if is_empty(r) else r


def ask_with_default_num(msg: str, default: int) -> int:
    r = ask(msg)
    if is_empty(r):
        return int(default)
    if can_be_int(r):
        return int(r)
    print("Invalid input, try again.")
    return ask_with_default_num(msg, default)


def ask_yn(msg: str, default: bool = False) -> bool:
    r = ask(msg)
    if is_empty(r):
        return default
    if not is_yes_or_no(r):
        print("Invalid input, try again.")
        return ask_yn(msg, default)
    return is_yes(r)


def ask_Yn(msg: str) -> bool:
    return ask_yn(msg, True)


def ask_yN(msg: str) -> bool:
    return ask_yn(msg, False)


def ask_no_empty(msg: str) -> str:
    r = ask(msg)
    return ask_no_empty(msg) if is_empty(r) else r


def confirm_proceed() -> None:
    if not ask_yN("Do you want to proceed? (y/N): "):
        print("Canceled.")
        sys.exit(0)



from abc import ABC, abstractmethod
from enum import Enum, auto

from ..file_path import PypjFilePath
from ..setting import PypjSetting


class TaskError(Exception):
    pass


class TaskStatus(Enum):
    TODO = auto()
    IN_PROGRESS = auto()
    DONE = auto()


class Task(ABC):
    def __init__(self, name: str, setting: PypjSetting, path: PypjFilePath) -> None:
        self.name: str = name
        self.status: TaskStatus = TaskStatus.TODO
        self.setting = setting
        self.path = path

    def done(self) -> None:
        self.status = TaskStatus.DONE

    def is_done(self) -> bool:
        return True if self.status is TaskStatus.DONE else False

    def done_check(self) -> None:
        if self.is_done():
            raise TaskError("Task is already done.")

    @abstractmethod
    def execute(self) -> None:
        pass



from enum import Enum


class Message(object):
    pass


class Emsg(Enum):
    OS_NOT_SUPPORTED = "Pypj only supports unix system."
    FAILED_GET_POETRY_VER = "Failed to get poetry version."
    FAILED_POETRY_NEW = "Failed to 'poetry new'."
    FAILED_POETRY_CONFIG = "Failed to configure poetry."
    FAILED_POETRY_ADD = "Failed to add package via poetry."
    RESOURCE_NOT_FOUND = "Resource does not exist. Please report this issue."
    PACKAGE_NOT_FOUND = "The target package does not exist."

    def __str__(self) -> str:
        return self.name


class PypjError(Exception):
    def __init__(self, message: Emsg):
        self.message = message.value
        super().__init__(self.message)



from enum import Enum


class Eenum(Enum):
    def __str__(self) -> str:
        return str(self.value)


class Formatter(Eenum):
    BLACK = "black"


class Linter(Eenum):
    FLAKE8 = "flake8"
    PFLAKE8 = "pyproject-flake8"


class ImportSorter(Eenum):
    ISORT = "isort"


class TypeChecker(Eenum):
    MYPY = "mypy"


class TestFramework(Eenum):
    PYTEST = "pytest"
    TOX = "tox"


class Plugin(Eenum):
    PYTEST_COV = "pytest-cov"
    PYTEST_MOCK = "pytest-mock"
    TOX_GH_ACTIONS = "tox-gh-actions"



import argparse
import sys

from .const import HELP


def args() -> None:
    parser = argparse.ArgumentParser(description="Modern python project builder")
    parser.add_argument("-v", "--version", action="store_true", help="Show version")
    args = parser.parse_args()
    if args.version:
        print(HELP)
        sys.exit(0)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypj', 'pypj.resources', 'pypj.task']

package_data = \
{'': ['*']}

install_requires = \
['single-source>=0.2.0,<0.3.0']

entry_points = \
{'console_scripts': ['pypj = pypj.main:main']}

setup_kwargs = {
    'name': 'pypj',
    'version': '0.6.1',
    'description': 'Python project initializer',
    'long_description': '![Pypj Logo](https://raw.githubusercontent.com/edge-minato/pypj/main/doc/img/logo.png)\n\n[![pypi version](https://img.shields.io/pypi/v/pypj.svg?style=flat)](https://pypi.org/pypi/pypj/)\n[![python versions](https://img.shields.io/pypi/pyversions/pypj.svg?style=flat)](https://pypi.org/pypi/pypj/)\n[![format](https://img.shields.io/pypi/format/pypj.svg?style=flat)](https://pypi.org/pypi/pypj/)\n[![license](https://img.shields.io/pypi/l/pypj.svg?style=flat)](https://github.com/edge-minato/pypj/blob/master/LICENSE)\n[![Unittest](https://github.com/edge-minato/pypj/actions/workflows/unittest.yml/badge.svg)](https://github.com/edge-minato/pypj/actions/workflows/unittest.yml)\n[![codecov](https://codecov.io/gh/edge-minato/pypj/branch/main/graph/badge.svg?token=YDZAMKUNS0)](https://codecov.io/gh/edge-minato/pypj)\n[![Code style](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black")\n[![Downloads](https://img.shields.io/pypi/dm/fbm.svg)](https://pypistats.org/packages/pypj)\n\n`Pypj` provides you an initialized modern python project. All the basic dev package installations, their configurations, and test workflows will be done, so we can focus on coding. All you have to do is install `poetry` and hit `pypj`, name your project.\n\n## What will be provided\n\nThe _"Modern"_ project settings `Pypj` suggests is following. We understand some developers prefer another tools, and you can remove or customize the packages to be installed.\n\n### Environment\n\n- Package manager: [`poetry`](https://github.com/python-poetry/poetry)\n- Formatter: [`black`](https://github.com/psf/black)\n- Linter: [`pflake8`](https://github.com/csachs/pyproject-flake8)\n- Type linter: [`mypy`](https://github.com/python/mypy)\n- Import formatter: [`isort`](https://github.com/PyCQA/isort)\n- Test framework:\n  - [`pytest`](https://github.com/pytest-dev/pytest)\n    - [`pytest-cov`](https://github.com/pytest-dev/pytest-cov)\n    - [`pytest-mock`](https://github.com/pytest-dev/pytest-mock)\n  - [`tox`](https://github.com/tox-dev/tox)\n    - [`tox-gh-actions`](https://github.com/ymyzk/tox-gh-actions)\n\n### Coding format\n\n- Max line length: `119` as default\n- Type hinting: `required`\n- And some detailed configures\n\n### Other features\n\n- Single filed configurations on `pyproject.toml`\n- Single sourced versioning: [`single-source`](https://github.com/rabbit72/single-source)\n- Command alias: [`make`](https://www.gnu.org/software/make/)\n- CI/CD\n  - unittest workflow\n\n### Directory structure\n\nDo you think the directory tree looks poor? Because all configurations are aggregated in `pyproject.toml`, we don\'t need any tool specific configuration files.\n\n```\n$ tree -a -L 1\nmy-package/\n├── .github\n├── .pre-commit-config.yaml\n├── .venv\n├── .vscode\n├── Makefile\n├── README.md\n├── my-package\n├── poetry.lock\n├── pyproject.toml\n└── tests\n```\n\n## Requirements\n\n- `python3`\n- `poetry` [[Installation guide](https://python-poetry.org/docs/#installation)]\n\n## Installation\n\n```sh\npip install pypj\n```\n\n## Usage\n\n```\n$ pypj\n\n┌─┐┬ ┬┌─┐┬\n├─┘└┬┘├─┘│    python : 3.8.5\n┴   ┴ ┴ └┘    poetry : 1.1.8\n\nPackage name: my-package\nDo you want to customize settings? (y/N): N\nDo you want to proceed? (y/N): y\nTask: Initialize package: my-package\n      Command: poetry new my-package ✨\n      Poetry new done 🚀\n      Command: poetry config virtualenvs.in-project true ✨\n      Command: poetry add -D black ✨\n      Command: poetry add -D pyproject-flake8 ✨\n      Command: poetry add -D mypy ✨\n      Command: poetry add -D isort ✨\n      Command: poetry add -D pytest ✨\n      Command: poetry add -D tox ✨\n      Command: poetry add -D pytest-cov ✨\n      Command: poetry add -D pytest-mock ✨\n      Command: poetry add -D tox-gh-actions ✨\n      Configure: __init__.py  ✨\n      Create : my-package ✨\nTask: Create README.md\n      Create : README.md ✨\nTask: Configure pyproject.toml settings\n      Write  : pyproject.toml ✨\nTask: Create github actions\n      Create : unittest.yml ✨\n      Create : publish.yml ✨\n      Create : dependabot.yml ✨\nTask: Configure vscode settings\n      Create : .vscode/settings.json ✨\nTask: Create makefile\n      Create : Makefile ✨\nTask: Configure pre-commit\n      Create : .pre-commit-config.yaml ✨\n\nComplete! 🚀\nLet\'s make the world better! ✨😋🐍🌎\n```\n\n## Example configurations on `pyproject.toml`\n\nWith default setting, this kind of `pyproject.toml` file will be generated.\n\n```toml\n[tool.poetry]\nname = "my-package"\nversion = "0.1.0"\ndescription = ""\nauthors = ["you <you@example.com>"]\n\n[tool.poetry.dependencies]\npython = "^3.8"\n\n[tool.poetry.dev-dependencies]\npytest = "^5.2"\nblack = "^21.8b0"\npyproject-flake8 = "^0.0.1-alpha.2"\nmypy = "^0.910"\nisort = "^5.9.3"\npytest-cov = "^2.12.1"\n\n[build-system]\nrequires = ["poetry-core>=1.0.0"]\nbuild-backend = "poetry.core.masonry.api"\n\n[tool.black]\nline-length = 119\nexclude = \'\'\'\n(\n    migrations\n    | .mypy_cache\n    | .pytest_cache\n    | .tox\n    | venv\n)\n\'\'\'\n\n[tool.flake8]\nmax-line-length = 119\nmax-complexity = 10\n\n[tool.mypy]\n# common\npython_version = 3.8\nshow_column_numbers = true\nshow_error_context = true\nignore_missing_imports = true\ncheck_untyped_defs = true\ndisallow_untyped_defs = true\n# warning\nwarn_return_any = true\nwarn_unused_configs = true\nwarn_redundant_casts = true\n\n[tool.isort]\nprofile = "black"\nline_length = 119\n```\n\n## Supported python versions\n\n- Supported: `3.7`, `3.8`, `3.9`\n- Is going to be supported: `3.10`\n- Not supported: `3.6` or less\n\n**NOTE**: According to [Status of Python branches](https://devguide.python.org/#status-of-python-branches), the EoL of Python 3.6 is `2021-12-23`.\n',
    'author': 'edge-minato',
    'author_email': 'edge.minato@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/edge-minato/pypj',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from ..const import INDENT, Emoji
from ..file_path import PypjFilePath
from ..setting import BLACK_SETTING, FLAKE8_SETTING, ISORT_SETTING, MYPY_SETTING, TOX_SETTING, PypjSetting
from .task import Task


class Pyproject(Task):
    def __init__(self, setting: PypjSetting, path: PypjFilePath) -> None:
        name = "Pyproject"
        super().__init__(name, setting, path)

    def execute(self) -> None:
        print("Task: Configure pyproject.toml settings")
        self.done_check()
        mll = self.setting.max_line_length
        pv = self.setting.python_version.short
        fmt = self.setting.formatter
        with self.path.pyproject.open(mode="a") as f:
            f.write(BLACK_SETTING.format(line_length=mll))
            f.write(FLAKE8_SETTING.format(line_length=mll))
            f.write(MYPY_SETTING.format(python_version=pv))
            f.write(ISORT_SETTING.format(formatter=fmt, line_length=mll))
            py3x_version = f"py{self.setting.python_version.major}{self.setting.python_version.minor}"
            package_dir = "src" if self.setting.use_src else self.setting.package_name
            f.write(TOX_SETTING.replace("PY3X_VERSION", py3x_version).replace("PACKAGE_DIR", package_dir))
        print(f"{INDENT}Write  : pyproject.toml {Emoji.OK}")
        self.done()



from ..const import INDENT, Emoji
from ..file_path import PypjFilePath
from ..resource import get_my_resource
from ..setting import PypjSetting
from .task import Task


class Vscode(Task):
    def __init__(self, setting: PypjSetting, path: PypjFilePath) -> None:
        name = "Vscode"
        super().__init__(name, setting, path)

    def execute(self) -> None:
        print("Task: Configure vscode settings")
        self.done_check()
        # get settings
        settings_jsonc = get_my_resource("settings.jsonc")
        settings_jsonc = settings_jsonc.replace("PACKAGE_NAME", self.setting.package_name)
        # mkdir .vscode
        self.path.vscode_dir.mkdir()
        # write settings
        with self.path.vscode_settings_json.open(mode="w") as f:
            f.write(settings_jsonc)
        print(f"{INDENT}Create : .vscode/settings.json {Emoji.OK}")
        self.done()



from ..const import INDENT, Emoji
from ..file_path import PypjFilePath
from ..resource import get_my_resource
from ..setting import PypjSetting
from .task import Task


class GithubActions(Task):
    def __init__(self, setting: PypjSetting, path: PypjFilePath) -> None:
        name = "GithubActions"
        super().__init__(name, setting, path)

    def execute(self) -> None:
        print("Task: Create github actions")
        self.done_check()
        # prepare directory
        self.path.github_dir.mkdir()
        self.path.github_workflow_dir.mkdir()
        # write workflows *.yml
        unittest = get_my_resource("unittest.yml")
        unittest = unittest.replace("PYTHON_VERSION", self.setting.python_version.short)
        publish = get_my_resource("publish.yml")
        publish = publish.replace("PYTHON_VERSION", self.setting.python_version.short)
        dependabot = get_my_resource("dependabot.yml")
        with self.path.wf_unittest.open(mode="w") as f:
            f.write(unittest)
        print(f"{INDENT}Create : unittest.yml {Emoji.OK}")
        with self.path.wf_publish.open(mode="w") as f:
            f.write(publish)
        print(f"{INDENT}Create : publish.yml {Emoji.OK}")
        with self.path.dependabot.open(mode="w") as f:
            f.write(dependabot)
        print(f"{INDENT}Create : dependabot.yml {Emoji.OK}")
        self.done()



from pathlib import Path

from .setting import PypjSetting


class PypjFilePath(object):
    def __init__(self, pwd: Path, setting: PypjSetting) -> None:
        root = pwd.resolve()
        self.root: Path = root  # as absolute path
        self.package_dir: Path = root.joinpath(setting.package_name)
        src_dir = "src" if setting.use_src else setting.package_name
        self.package_src_dir: Path = self.package_dir.joinpath(src_dir)
        self.package_init: Path = self.package_src_dir.joinpath("__init__.py")
        self.pyproject: Path = self.package_dir.joinpath("pyproject.toml")
        self.vscode_dir: Path = self.package_dir.joinpath(".vscode")
        self.vscode_settings_json: Path = self.vscode_dir.joinpath("settings.json")
        self.makefile: Path = self.package_dir.joinpath("Makefile")
        self.readme_md: Path = self.package_dir.joinpath("README.md")
        self.readme_rst: Path = self.package_dir.joinpath("README.rst")
        self.github_dir: Path = self.package_dir.joinpath(".github")
        self.github_workflow_dir: Path = self.github_dir.joinpath("workflows")
        self.wf_unittest: Path = self.github_workflow_dir.joinpath("unittest.yml")
        self.wf_publish: Path = self.github_workflow_dir.joinpath("publish.yml")
        self.dependabot: Path = self.github_dir.joinpath("dependabot.yml")
        self.precommit: Path = self.package_dir.joinpath(".pre-commit-config.yaml")

    def print(self) -> None:
        print(vars(self))
